package com.sun.xml.internal.bind.v2.runtime.unmarshaller;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import javax.xml.namespace.NamespaceContext;

public final class InterningXmlVisitor implements XmlVisitor
{
    private final XmlVisitor next;
    private final InterningXmlVisitor.AttributesImpl attributes;
    
    public InterningXmlVisitor(XmlVisitor next) {
        this.attributes = new InterningXmlVisitor.AttributesImpl();
        this.next = next;
    }
    
    @Override
    public void startDocument(LocatorEx locator, NamespaceContext nsContext) throws SAXException {
        this.next.startDocument(locator, nsContext);
    }
    
    @Override
    public void endDocument() throws SAXException {
        this.next.endDocument();
    }
    
    @Override
    public void startElement(TagName tagName) throws SAXException {
        this.attributes.setAttributes(tagName.atts);
        tagName.atts = this.attributes;
        tagName.uri = intern(tagName.uri);
        tagName.local = intern(tagName.local);
        this.next.startElement(tagName);
    }
    
    @Override
    public void endElement(TagName tagName) throws SAXException {
        tagName.uri = intern(tagName.uri);
        tagName.local = intern(tagName.local);
        this.next.endElement(tagName);
    }
    
    @Override
    public void startPrefixMapping(String prefix, String nsUri) throws SAXException {
        this.next.startPrefixMapping(intern(prefix), intern(nsUri));
    }
    
    @Override
    public void endPrefixMapping(String prefix) throws SAXException {
        this.next.endPrefixMapping(intern(prefix));
    }
    
    @Override
    public void text(CharSequence pcdata) throws SAXException {
        this.next.text(pcdata);
    }
    
    @Override
    public UnmarshallingContext getContext() {
        return this.next.getContext();
    }
    
    @Override
    public XmlVisitor.TextPredictor getPredictor() {
        return this.next.getPredictor();
    }
    
    private static String intern(String s) {
        if (s == null) {
            return null;
        }
        return s.intern();
    }
    
    static String access$100(String x0) {
        return intern(x0);
    }
    
    private static class AttributesImpl implements Attributes
    {
        private Attributes core;
        
        void setAttributes(Attributes att) {
            this.core = att;
        }
        
        @Override
        public int getIndex(String qName) {
            return this.core.getIndex(qName);
        }
        
        @Override
        public int getIndex(String uri, String localName) {
            return this.core.getIndex(uri, localName);
        }
        
        @Override
        public int getLength() {
            return this.core.getLength();
        }
        
        @Override
        public String getLocalName(int index) {
            return InterningXmlVisitor.access$100(this.core.getLocalName(index));
        }
        
        @Override
        public String getQName(int index) {
            return InterningXmlVisitor.access$100(this.core.getQName(index));
        }
        
        @Override
        public String getType(int index) {
            return InterningXmlVisitor.access$100(this.core.getType(index));
        }
        
        @Override
        public String getType(String qName) {
            return InterningXmlVisitor.access$100(this.core.getType(qName));
        }
        
        @Override
        public String getType(String uri, String localName) {
            return InterningXmlVisitor.access$100(this.core.getType(uri, localName));
        }
        
        @Override
        public String getURI(int index) {
            return InterningXmlVisitor.access$100(this.core.getURI(index));
        }
        
        @Override
        public String getValue(int index) {
            return this.core.getValue(index);
        }
        
        @Override
        public String getValue(String qName) {
            return this.core.getValue(qName);
        }
        
        @Override
        public String getValue(String uri, String localName) {
            return this.core.getValue(uri, localName);
        }
    }
}
